//this is the data grid provider
dojo.provide("dtacos.grid.TableDataProvider");

//
// Template path ../../../../src/js/dtacos/grid/templates/
//
//
// DataGridDataProvider spec taken from:
// http://whatwg.org/specs/web-apps/current-work/#scs-data
//
dojo.require("dojo.widget.*");
dojo.require("dojo.event.*");
dojo.require("dojo.fx.*");
dojo.require("dojo.html");

//
// wtf am I doing here??
dtacos.grid.DataGridDataProvider = function() {
}

dojo.inherits( dtacos.grid.TableDataProvider, 
				dtacos.grid.DataGridDataProvider);

dtacos.grid.TableDataProvider = function(node, grid) {
	this.domNode = node;
	this.initialized = false;
	this.grid = grid;
	
	//create our table logic
	this.initialize();
}

dojo.lang.extend(dtacos.grid.TableDataProvider, {
	
	columnCaptions: [],
	cols: [],
	rows: [],
	rowIndex: [],
	ASC: "asc",
	DESC: "desc",
	
	initialize: function() {
		if (!this.domNode || !this.domNode.childNodes) {
			dojo.debug("WARN TableDataProvider: No data found. returning.");
			return;	
		}
		
		//parsing data
		var children = this.domNode.childNodes;
		var bfound = false;
		for (var i=0; i < children.length; i++) {
			if (children[i].nodeType != 1) continue;
			
			var tag = dojo.dom.getTagName(children[i]);
			if (tag == "thead") {
				for (var h=0; h < children[i].childNodes.length; h++) {
					if (children[i].childNodes[h].nodeType != 1) continue;
					
					if (dojo.dom.getTagName(children[i].childNodes[h]) == "tr") {
						this.addCaptions(children[i].childNodes[h].childNodes);
						break;
					}
				}
			}
			if (tag == "tbody") {
				bfound = true;
				this.addRows(children[i].childNodes);
			}
		}
		//spec says first find tbody rows, but
		//if no tbody is found, use tr that are children of table
		if (!bfound) {
			this.addRows(children);
		}
		
		
		//spec says must always return at least 1
		if (this.cols.length <= 0) {
			this.cols[0] = {};
		}
		
		//spec says caption text empty if no captions found
		if (this.columnCaptions.length == 0) {
			for (var i=0; i < this.rows.length; i++) {
				this.columnCaptions.push("");
			}
		}
		
		//hopefully this should never happen
		if (this.columnCaptions.length != this.cols.length) {
			dojo.raise("ERROR TableDataProvider column length != columnCaptions "
			+ this.cols.length + " != " + this.columnCaptions.length);
		}
		
		//just in case
		if (this.rowIndex.length != this.rows.length) {
			dojo.raise("ERROR rowIndex not same as rows length");
		}
		
		this.initialized = true;
		this.grid.updateEverything();
  	},
  	
  	addRows: function(bnodes) {
  		//Parse tr children of body
		for (var b=0; b < bnodes.length; b++) {
			if (bnodes[b].nodeType != 1) continue;
			if (dojo.dom.getTagName(bnodes[b]) == "tr") {
				this.addRowData(bnodes[b]);
			}
		}
  	},
  	
  	addRowData: function(node) {
  		var rowref = {};
  		rowref["domNode"] = node;
  		var td = dojo.dom.firstElement(node);
  		if (td) {
  			var imgn = dojo.dom.firstElement(td);
  			if (imgn && dojo.dom.getTagName(imgn) == "img") {
  				rowref["imguri"] = dojo.html.getAttribute(imgn, "src");
  			}
  		}
  		if (!rowref["imguri"]) rowref["imguri"] = "";
  		
  		this.rows.push(rowref);
  		this.rowIndex[this.rows.length - 1] = this.rows.length - 1;
		
  		//Spec says first valid rows count of td elements
  		//determines the column count
  		if (this.cols.length == 0) {
  			var colCount = 0;
  			for (var i=0; i < node.childNodes.length; i++) {
  				if (node.childNodes[i].nodeType != 1) continue;
  				if (dojo.dom.getTagName(node.childNodes[i]) == "td") {
  					this.cols[colCount] = {};
  					colCount++;
  				}
  			}
  		}
  		
  		//build row column structures
  		rowref["cols"] = [];
  		for (var i=0; i < node.childNodes.length; i++) {
			if (node.childNodes[i].nodeType != 1) continue;
			if (dojo.dom.getTagName(node.childNodes[i]) == "td") {
				var col = {};
				col["domNode"] = node.childNodes[i];
				rowref["cols"].push(col);
			}
  		}
  	},
  	
  	addCaptions: function(nodes) {
  		for (var i=0; i < nodes.length; i++) {
  			if (nodes[i].nodeType != 1) continue;
  			if (dojo.dom.getTagName(nodes[i]) == "th") {
  				this.columnCaptions.push(nodes[i].textContent);
  			}
  		}
  	},
  	
  	rowValue: function(index) {
  		return this.rows[this.rowIndex[index]]
  	},
  	
  	getRowCount: function() {
  		return this.rowIndex.length;
  	},
  	
  	getColumnCount: function() {
  		return this.cols.length;
  	},
  	
  	getCaptionText: function(columnIndex) {
  		return this.columnCaptions[columnIndex];
  	},
  	
  	getCaptionClasses: function(columnIndex, classToken) {
  		return null;
  	},
  	
  	getRowParent: function(rowIndex) {
  		return -1;
  	},
  	
  	getRowImage: function(rowIndex) {
  		return this.rowValue(rowIndex)["imguri"];
  	},
  	
  	getRowMenu: function(rowIndex) {
  		return null;
  	},
  	
  	getRowClasses: function(rowIndex, classToken) {
  		return null;
  	},
  	
  	getCellData: function(rowIndex, columnIndex) {
  		return this.rowValue(rowIndex)["cols"][columnIndex]["domNode"].textContent;
  	},
  	
  	getCellClasses: function(rowIndex, columnIndex, classToken) {
  		return null;
  	},
  	
  	toggleRowOpenState: function(rowIndex) {
  		return 1;
  	},
  	
  	toggleColumnSortState: function(columnIndex) {
  		return 1;
  	},
  	
  	setCellCheckedState: function(rowIndex, columnIndex, state) {
  		return 1;
  	},
  	
  	cycleCell: function(rowIndex, columnIndex) {
  		return 1;
  	},
  	
  	editCell: function(rowIndex, columnIndex, data) {
  		return 1;
  	}
});